@@ -10,6 +10,7 @@ use serde::Deserialize;
 use serde::de::{Deserializer, Error};
 
 use crate::core::{EmptyResult, GenericResult};
+use crate::currency::Cash;
 use crate::formatting::format_date;
 use crate::localities::Jurisdiction;
 use crate::time::{Date, DateTime, DateOptTime, deserialize_date_opt_time};
@@ -48,6 +49,14 @@ impl CorporateAction {
 #[cfg_attr(test, derive(PartialEq))]
 #[serde(tag = "type", rename_all="kebab-case")]
 pub enum CorporateActionType {
+    #[serde(skip)]
+    Liquidation {
+        quantity: Decimal,
+        price: Decimal,
+        volume: Decimal,
+        currency: String,
+    },
+
     // See https://github.com/KonishchevDmitry/investments/issues/29 for details
     Rename {
         new_symbol: String,
@@ -204,6 +213,17 @@ pub fn process_corporate_actions(statement: &mut BrokerStatement) -> EmptyResult
 
 fn process_corporate_action(statement: &mut BrokerStatement, action: CorporateAction) -> EmptyResult {
     match action.action {
+        CorporateActionType::Liquidation {quantity, price, volume, ref currency} => {
+            let price = Cash::new(currency, price);
+            let volume = Cash::new(currency, volume);
+            let commission = Cash::zero(currency);
+
+            statement.stock_sells.push(StockSell::new_trade(
+                &action.symbol, quantity, price, volume, commission,
+                action.time, action.execution_date(), false));
+            statement.sort_and_validate_stock_sells()?;
+        },
+
         CorporateActionType::Rename {ref new_symbol} => {
             statement.rename_symbol(&action.symbol, new_symbol, Some(action.time)).map_err(|e| format!(
                 "Failed to process {} -> {} rename corporate action: {}",
