@@ -1,15 +1,16 @@
 #[cfg(test)] use csv::StringRecord;
 use lazy_static::lazy_static;
-use regex::Regex;
+use regex::{Regex, Captures};
 
 use crate::broker_statement::corporate_actions::{CorporateAction, CorporateActionType, StockSplitRatio};
 use crate::core::{EmptyResult, GenericResult};
+use crate::currency::Cash;
 use crate::formatting::format_date;
 #[cfg(test)] use crate::types::{Date, DateTime, Decimal};
 use crate::util::{self, DecimalRestrictions};
 
 use super::StatementParser;
-use super::common::{self, Record, RecordParser, SecurityID, parse_symbol};
+use super::common::{self, Record, RecordParser, SecurityID, check_volume, parse_symbol};
 #[cfg(test)] use super::common::RecordSpec;
 
 pub struct CorporateActionsParser {
@@ -22,6 +23,12 @@ impl RecordParser for CorporateActionsParser {
     }
 
     fn parse(&mut self, _parser: &mut StatementParser, record: &Record) -> EmptyResult {
+        // Supplementary record for corporate actions like delisting and liquidation
+        // (see https://github.com/KonishchevDmitry/investments/issues/69)
+        if record.get_value("Report Date")? == "Closed Lot:" {
+            return Ok(());
+        }
+
         self.corporate_actions.push(parse(record)?);
         Ok(())
     }
@@ -79,7 +86,7 @@ fn parse(record: &Record) -> GenericResult<CorporateAction> {
     let description = description.as_ref();
 
     lazy_static! {
-        static ref REGEX: Regex = Regex::new(&format!(concat!(
+        static ref GENERIC_REGEX: Regex = Regex::new(&format!(concat!(
             r"^(?P<symbol>{symbol}) ?\({id}\) ",
             r"(?P<action>Spinoff|Split|Stock Dividend|Subscribable Rights Issue) ",
             r"(?:{id} )?(?P<to>[1-9]\d*) for (?P<from>[1-9]\d*) ",
@@ -93,18 +100,50 @@ fn parse(record: &Record) -> GenericResult<CorporateAction> {
 
             symbol=common::STOCK_SYMBOL_REGEX, old_suffix=regex::escape(common::OLD_SYMBOL_SUFFIX),
             id=SecurityID::REGEX)).unwrap();
+
+        static ref LIQUIDATION_REGEX: Regex = Regex::new(&format!(concat!(
+            r"^(?P<symbol>{symbol}) ?\({id}\) ",
+            r"(?P<action>Merged\(Liquidation\)) ",
+            r"FOR (?P<currency>[A-Z]{{3}}) (?P<price>[0-9.]+) PER SHARE ",
+            r"\((?P<other_symbol>{symbol}), [^,)]+, {id}\)$"),
+            symbol=common::STOCK_SYMBOL_REGEX, id=SecurityID::REGEX)).unwrap();
     }
 
-    let error = || Err!("Unsupported corporate action: {:?}", description);
-    let captures = match REGEX.captures(description) {
-        Some(captures) => captures,
-        None => return error(),
-    };
+    let captures: Captures = GENERIC_REGEX.captures(description)
+        .or_else(|| LIQUIDATION_REGEX.captures(description))
+        .ok_or_else(|| format!("Unsupported corporate action: {:?}", description))?;
 
     let symbol = parse_symbol(captures.name("symbol").unwrap().as_str())?;
     let other_symbol = parse_symbol(captures.name("other_symbol").unwrap().as_str())?;
+    let error = || Err!("Unsupported corporate action: {:?}", description);
 
     let action = match captures.name("action").unwrap().as_str() {
+        "Merged(Liquidation)" => {
+            if other_symbol != symbol {
+                return error();
+            }
+
+            let currency = record.get_value("Currency")?;
+            let volume = record.parse_amount("Proceeds", DecimalRestrictions::PositiveOrZero)?;
+            let quantity = -record.parse_quantity("Quantity", DecimalRestrictions::StrictlyNegative)?;
+
+            let price = util::parse_decimal(
+                captures.name("price").unwrap().as_str(),
+                DecimalRestrictions::PositiveOrZero)?;
+
+            let price_currency = captures.name("currency").unwrap().as_str();
+            if price_currency != currency {
+                return Err!("Price and volume currency mismatch: {} vs {}", price_currency, currency);
+            }
+
+            check_volume(quantity, Cash::new(currency, price), Cash::new(currency, volume))?;
+
+            CorporateActionType::Liquidation {
+                quantity, price, volume,
+                currency: currency.to_owned(),
+            }
+        },
+
         "Spinoff" => {
             let quantity = record.parse_quantity("Quantity", DecimalRestrictions::StrictlyPositive)?;
             let currency = record.get_value("Currency")?.to_owned();
@@ -206,6 +245,60 @@ mod tests {
     use super::*;
     use rstest::rstest;
 
+    #[test]
+    fn liquidation() {
+        test_parsing(&[
+            "Stocks", "USD", "2021-11-03", "2021-11-02, 20:25:00",
+            "CHL(US16941M1099) Merged(Liquidation) FOR USD 30.20446 PER SHARE (CHL, CHINA MOBILE LTD-SPON ADR, US16941M1099)",
+            "-10", "302.0446", "-275.1", "3.012343", "",
+        ], CorporateAction {
+            time: date_time!(2021, 11, 2, 20, 25, 00).into(),
+            report_date: Some(date!(2021, 11, 3)),
+
+            symbol: s!("CHL"),
+            action: CorporateActionType::Liquidation {
+                quantity: dec!(10),
+                price: dec!(30.20446),
+                volume: dec!(302.0446),
+                currency: s!("USD"),
+            },
+        });
+    }
+
+    #[test]
+    fn spinoff() {
+        test_parsing(&[
+            "Stocks", "USD", "2020-11-17", "2020-11-16, 20:25:00",
+            "PFE(US7170811035) Spinoff  124079 for 1000000 (VTRS, VIATRIS INC-W/I, US92556V1061)",
+            "9.3059", "0", "0", "0", "",
+        ], CorporateAction {
+            time: date_time!(2020, 11, 16, 20, 25, 00).into(),
+            report_date: Some(date!(2020, 11, 17)),
+
+            symbol: s!("PFE"),
+            action: CorporateActionType::Spinoff {
+                symbol: s!("VTRS"),
+                quantity: dec!(9.3059),
+                currency: s!("USD"),
+            },
+        });
+    }
+
+    #[test]
+    fn stock_dividend() {
+        test_parsing(&[
+            "Stocks", "USD", "2020-07-17", "2020-07-17, 20:20:00",
+            "TEF (US8793822086) Stock Dividend US8793822086 416666667 for 10000000000 (TEF, TELEFONICA SA-SPON ADR, US8793822086)",
+            "1", "0", "4.73", "0", "",
+        ], CorporateAction {
+            time: date_time!(2020, 7, 17, 20, 20, 0).into(),
+            report_date: Some(date!(2020, 7, 17)),
+
+            symbol: s!("TEF"),
+            action: CorporateActionType::StockDividend {quantity: dec!(1)},
+        });
+    }
+
     #[rstest(record, symbol, time, report_date, to, from, from_change, to_change,
         case(&[
             "Stocks", "USD", "2020-08-31", "2020-08-28, 20:25:00",
@@ -246,40 +339,6 @@ mod tests {
         });
     }
 
-    #[test]
-    fn stock_dividend() {
-        test_parsing(&[
-            "Stocks", "USD", "2020-07-17", "2020-07-17, 20:20:00",
-            "TEF (US8793822086) Stock Dividend US8793822086 416666667 for 10000000000 (TEF, TELEFONICA SA-SPON ADR, US8793822086)",
-            "1", "0", "4.73", "0", "",
-        ], CorporateAction {
-            time: date_time!(2020, 7, 17, 20, 20, 0).into(),
-            report_date: Some(date!(2020, 7, 17)),
-
-            symbol: s!("TEF"),
-            action: CorporateActionType::StockDividend {quantity: dec!(1)},
-        });
-    }
-
-    #[test]
-    fn spinoff() {
-        test_parsing(&[
-            "Stocks", "USD", "2020-11-17", "2020-11-16, 20:25:00",
-            "PFE(US7170811035) Spinoff  124079 for 1000000 (VTRS, VIATRIS INC-W/I, US92556V1061)",
-            "9.3059", "0", "0", "0", "",
-        ], CorporateAction {
-            time: date_time!(2020, 11, 16, 20, 25, 00).into(),
-            report_date: Some(date!(2020, 11, 17)),
-
-            symbol: s!("PFE"),
-            action: CorporateActionType::Spinoff {
-                symbol: s!("VTRS"),
-                quantity: dec!(9.3059),
-                currency: s!("USD"),
-            },
-        });
-    }
-
     #[rstest(record, symbol, time, report_date,
         case(&[
             "Stocks", "USD", "2021-06-18", "2021-06-16, 20:25:00",
